package com.softslate.commerce.businessobjects.events;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.businessobjects.importexport.QboConnectionProcessor;
import com.softslate.commerce.businessobjects.importexport.QboOrderProcessor;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.daos.order.OrderDAO;

public class QboOrderTriggerImpl extends BaseTrigger implements QboOrderTrigger {

	protected Order order;
	protected QboOrderProcessor qboProcessor;
	protected QboConnectionProcessor qboConnectionProcessor;

	@Override
	public boolean getExecuteInSeparateThread() {
		return true;
	}

	@Override
	public void processNewOrder(NewOrderEvent noe) throws Exception {
		initializeFromEvent(noe);
		if (qboProcessor != null
				&& "NewOrderPlaced".equals(noe.getCartProcessor().getSettings().getValue("qboSyncOrdersToQbo"))) {
			log.info("Syncing new order upon completion: " + order.getOrderNumber());

			// Calls processTrigger below, after BaseTrigger optionally creates
			// a new thread or sets up exceptions to be swallowed.
			process();
		} else {
			log.info("QBO new order sync is disabled.");
		}
	}

	private void initializeFromEvent(NewOrderEvent noe) throws Exception {
		BusinessObjectFactory bof = noe.getCartProcessor().getBusinessObjectFactory();
		order = noe.getCartProcessor().getUser().getOrder();
		bof.getDaoFactory().createDAO(OrderDAO.class).initializeOrder(order);
		qboProcessor = bof.createObject(QboOrderProcessor.class);
		qboConnectionProcessor = bof.createObject(QboConnectionProcessor.class);
	}

	@Override
	public void processChangedOrder(ChangedOrderEvent coe) throws Exception {
		initializeFromEvent(coe);
		Order order = coe.getOrder();
		Map originalParameters = coe.getOriginalParameters();
		Settings settings = coe.getBusinessProcessor().getSettings();
		String qboOrderSyncStatusChange = (String) settings.getValue("qboOrderSyncStatusChange");
		if (qboProcessor != null
				&& "OrderStatusChange".equals(settings.getValue("qboSyncOrdersToQbo"))
				&& order != null && originalParameters != null && originalParameters.get("status") != null
				&& !originalParameters.get("status").equals(qboOrderSyncStatusChange)
				&& order.getStatus() != null && order.getStatus().equals(qboOrderSyncStatusChange)) {
			log.info("Syncing order changing to: " + qboOrderSyncStatusChange + " status: "
					+ order.getOrderNumber());

			// Calls processTrigger below, after BaseTrigger optionally creates
			// a new thread or sets up exceptions to be swallowed.
			process();
		} else {
			log.info("QBO order changing to Shipped sync is disabled.");
		}
	}

	private void initializeFromEvent(ChangedOrderEvent coe) throws Exception {
		BusinessObjectFactory bof = coe.getBusinessProcessor().getBusinessObjectFactory();
		order = coe.getOrder();
		bof.getDaoFactory().createDAO(OrderDAO.class).initializeOrder(order);
		qboProcessor = bof.createObject(QboOrderProcessor.class);
	}

	@Override
	public void processTrigger() throws Exception {
		try {
			Map results = qboProcessor.syncOrder(order);
			log.info("QBO order sync results for order " + order.getOrderNumber() + ": " + results);
			if (results != null && results.get("error") != null) {
				qboConnectionProcessor.sendQboConnectionWarningEmail(order, results.get("error").toString());
			}
		} catch (Exception e) {

			// Send a warning email on any exception from the trigger; otherwise it'll be invisible.
			qboConnectionProcessor.sendQboConnectionWarningEmail(order,
					"Error syncing order " + order.getOrderNumber() + " to QBO: " + e.toString());
			throw e;
		}
	}

}
