package com.softslate.commerce.businessobjects.product;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.product.ProductGatewayDAO;
import com.softslate.commerce.daos.product.ProductSettingDAO;
import com.softslate.commerce.daos.product.ProductSettingGatewayDAO;

/**
 * Processes business logic having to do with product settings. Created in the Struts layer in response to requests for
 * viewing, editing, adding, and deleting product settings.
 * <p>
 * <code>BasicProductSettingProcessor</code> is the default implementation of the <code>ProductSettingProcessor</code>
 * interface for the application.
 * 
 * @author David Tobey
 */
public class BasicProductSettingProcessor extends BaseBusinessProcessor implements ProductSettingProcessor {

	static Log log = LogFactory.getLog(BasicProductSettingProcessor.class);

	public Map loadProductSettingsAndCount(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading ProductSettings and count");
		ProductSettingGatewayDAO gatewayDAO = getDaoFactory().createDAO(ProductSettingGatewayDAO.class);
		return gatewayDAO.loadProductSettingsAndCount(parameters);
	}

	public Map addProductSetting(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName() + ": product setting: " + code);
		}
		Map results = new HashMap();

		// Look up the product - support using an productCode to refer to the parent
		ProductProcessor productProcessor = getBusinessObjectFactory().createObject(ProductProcessor.class);
		Product product = null;
		if (parameters.get("productID") != null) {
			product = productProcessor.loadProductFromID(parameters);
		}
		if (product == null && parameters.get("productCode") != null) {
			product = productProcessor
					.loadProductFromCode(Collections.singletonMap("code", parameters.get("productCode")));
		}
		if (product == null) {
			results.put("resultCode", "NoParentFound");
			return results;
		}
		parameters.put("productID", product.getProductID());

		// Check that another productSetting with the same code doesn't exist
		ProductSetting productSetting = getBusinessObjectFactory().createObject(ProductSetting.class);
		BeanUtils.copyProperties(productSetting, parameters);
		results = validateProductSetting(productSetting);
		if (results != null && results.get("resultCode") != null && !results.get("resultCode").equals("0")) {
			return results;
		}

		// Insert the productSetting
		productSetting.setProduct(product);
		ProductSettingDAO dao = (ProductSettingDAO) getDaoFactory().createDAO("productSettingDAOImplementer");
		dao.setProductSetting(productSetting);
		dao.insertProductSetting(true);
		results.put("newRecord", dao.getProductSetting());
		return results;
	}

	public ProductSetting loadProductSettingFromID(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading product setting from id");
		ProductSetting productSetting = (ProductSetting) getBusinessObjectFactory()
				.createObject("productSettingImplementer");
		ProductSettingDAO dao = getDaoFactory().createDAO(ProductSettingDAO.class);
		BeanUtils.copyProperties(productSetting, parameters);
		dao.setProductSetting(productSetting);
		dao.loadProductSettingFromID();
		return dao.getProductSetting();
	}

	public Map editProductSetting(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName() + ": Editing product setting: " + code);
		}
		Map results = new HashMap();

		// Check that another product setting with the same code doesn't exist
		// TODO support this even if productID is not provided - find the productID first from the productSettingID?
		if (parameters.get("productSettingID") != null && parameters.get("productID") != null) {
			ProductSetting productSetting = getBusinessObjectFactory().createObject(ProductSetting.class);
			BeanUtils.copyProperties(productSetting, parameters);
			results = validateProductSetting(productSetting);
			if (results != null && results.get("resultCode") != null && !results.get("resultCode").equals("0")) {
				return results;
			}
		}

		// Load the productSetting
		ProductSetting productSetting = null;
		if (parameters.get("productSettingID") != null) {
			productSetting = loadProductSettingFromID(parameters);
		}
		if (productSetting == null && parameters.get("code") != null) {
			productSetting = loadProductSettingFromCode(parameters);
		}

		// Update the productSetting
		BeanUtils.copyProperties(productSetting, parameters);

		Product product = getBusinessObjectFactory().createObject(Product.class);
		product.setProductID(productSetting.getProductID());
		productSetting.setProduct(product);
		ProductSettingDAO dao = (ProductSettingDAO) getDaoFactory().createDAO("productSettingDAOImplementer");
		dao.setProductSetting(productSetting);
		dao.updateProductSetting(true);
		results.put("updatedRecord", dao.getProductSetting());
		return results;
	}

	public Map validateProductSetting(ProductSetting productSetting) throws Exception {
		Map results = new HashMap();
		ProductSettingDAO dao = (ProductSettingDAO) getDaoFactory().createDAO("productSettingDAOImplementer");
		dao.setProductSetting(productSetting);
		dao.loadProductSettingFromCode();
		if (dao.getProductSetting() != null) {
			dao.getDaoFactory().getSession().evict(dao.getProductSetting());
		}
		boolean insertingButExists = productSetting.getProductSettingID() == 0 && dao.getProductSetting() != null
				&& dao.getProductSetting().getProductSettingID() > 0;
		boolean updatingButExists = productSetting.getProductSettingID() > 0 && dao.getProductSetting() != null
				&& productSetting.getProductSettingID() != dao.getProductSetting().getProductSettingID();
		if (insertingButExists || updatingButExists) {
			results.put("resultCode", "DuplicateRecord");
			results.put("duplicateField", "code");
			results.put("existingRecord", dao.getProductSetting());
		}
		return results;
	}

	public ProductSetting loadProductSettingFromCode(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading ProductSetting from code");

		// Look up the product - support using a productCode to refer to the parent
		ProductProcessor productProcessor = (ProductProcessor) getBusinessObjectFactory()
				.createObject(ProductProcessor.class);
		Product product = null;
		if (parameters.get("productID") == null && parameters.get("productCode") != null) {
			product = productProcessor
					.loadProductFromCode(Collections.singletonMap("code", parameters.get("productCode")));
			if (product == null) {
				log.error("Parent product not found.");
				return null;
			}
			parameters.put("productID", product.getProductID());
		}

		// Load the ProductSetting
		ProductSetting productSetting = (ProductSetting) getBusinessObjectFactory()
				.createObject("productSettingImplementer");
		ProductSettingDAO dao = (ProductSettingDAO) getDaoFactory().createDAO("productSettingDAOImplementer");
		BeanUtils.copyProperties(productSetting, parameters);
		dao.setProductSetting(productSetting);
		dao.loadProductSettingFromCode();
		return dao.getProductSetting();
	}

	public void deleteProductSetting(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName() + ": Deleting product setting: " + code);
		}

		// Load the product setting by productSettingID or code & productCode/productID
		ProductSetting productSetting = null;
		if (parameters.get("productSettingID") != null) {
			productSetting = loadProductSettingFromID(parameters);
		}
		if (productSetting == null && parameters.get("code") != null) {
			productSetting = loadProductSettingFromCode(parameters);
		}

		// Delete it
		ProductSettingDAO dao = getDaoFactory().createDAO(ProductSettingDAO.class);
		dao.setProductSetting(productSetting);
		dao.deleteProductSetting(true);
		return;
	}

	public int loadMaxProductSettingID() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			log.info(getUser().getAdministrator().getUserName() + ": Loading max product setting ID.");
		}
		ProductGatewayDAO gatewayDAO = getDaoFactory().createDAO(ProductGatewayDAO.class);
		return gatewayDAO.loadMaxProductSettingID();
	}
}
