package com.softslate.commerce.businessobjects.shipping;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.order.OrderShippingRule;
import com.softslate.commerce.daos.shipping.ShippingMethodGatewayDAO;

/**
 * Loads shipping options and processes shipping charges in the store. This
 * implementation looks up the shipping methods and rates configured in the
 * <code>sscShippingMethod</code> and <code>sscShippingRates</code> tables,
 * which provide for number of table-based shipping methods.
 * <p>
 * <code>BasicShippingProcessor</code> is the default implementation of the
 * <code>ShippingProcessor</code> interface for the application.
 * 
 * @author David Tobey
 * @see com.softslate.commerce.businessobjects.shipping.ShippingProcessor
 */
public class BasicShippingProcessor extends BaseShippingProcessor implements
		ShippingProcessor {

	static Log log = LogFactory.getLog(BasicShippingProcessor.class);

	public Collection loadShippingOptions() throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading shipping options.");
		Collection methodsAndRates = loadActiveMethodsAndRates();
		return computeShippingOptions(methodsAndRates);
	}

	public int processShipping() throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing shipping.");

		// Nothing to be done apart from the processing in BaseShippingProcessor
		return 0;
	}

	public Collection loadActiveMethodsAndRates() throws Exception {
		ShippingMethodGatewayDAO dao =  getDaoFactory()
				.createDAO(ShippingMethodGatewayDAO.class);
		return dao.loadActiveMethodsAndRates();
	}

	public Collection loadActiveRatesByMethodCode(String code) throws Exception {
		ShippingMethodGatewayDAO dao =  getDaoFactory()
				.createDAO(ShippingMethodGatewayDAO.class);
		return dao.loadActiveRatesByMethodCode(code);
	}

	public Collection loadContingencyShippingOption() throws Exception {
		Collection options = null;
		if (contingencyShippingMethodCodeDefined()) {
			String contingencyShippingMethodCode = (String) getSettings().getValue("contingencyShippingMethodCode");
			log.info("Going with contingency method " + contingencyShippingMethodCode);
			Collection methodsAndRates = loadRatesByMethodCode(contingencyShippingMethodCode);
			options = computeShippingOptions(methodsAndRates);
		}
		return options;
	}

	public boolean contingencyShippingMethodCodeDefined() {
		String contingencyShippingMethodCode = (String) getSettings().getValue("contingencyShippingMethodCode");
		return contingencyShippingMethodCode != null && contingencyShippingMethodCode.length() > 0;
	}

	public Collection loadRatesByMethodCode(String code) throws Exception {
		ShippingMethodGatewayDAO dao = (ShippingMethodGatewayDAO) getDaoFactory().createDAO(
				"shippingMethodGatewayDAOImplementer");
		return dao.loadRatesByMethodCode(code);
	}

	public Collection computeShippingOptions(Collection methodsAndRates)
			throws Exception {

		double totalWeight = 0;
		if (getUser().getOrder() != null
				&& getUser().getOrder().getOrderShippingRules() != null
				&& !getUser().getOrder().getOrderShippingRules().isEmpty()) {
			Iterator i = getUser().getOrder().getOrderShippingRules()
					.iterator();
			while (i.hasNext()) {
				OrderShippingRule osr = (OrderShippingRule) i.next();
				if (osr.getRuleType().equals("[package]"))
					totalWeight += osr.getPkgWeight().doubleValue();
			}
		}

		Collection options = new ArrayList();
		Iterator i = methodsAndRates.iterator();
		int previousMethodID = 0;
		Map option = null;
		double rawPrice = 0;
		double minimum = 0;
		while (i.hasNext()) {
			ShippingMethod method = (ShippingMethod) i.next();
			if (previousMethodID == method.getShippingMethodID()) {
				continue;
			}
			previousMethodID = method.getShippingMethodID();
			if (log.isDebugEnabled())
				log.debug("methodType: " + method.getMethodType());
			// Skip if this is the gift-certificates-only method but there are
			// no gift certificates
			if (method.getCode().equals("gift_certificates_only")
					&& !orderHasGiftCertificatesOnly(getUser().getOrder())) {
				continue;
			}
			boolean priceFound = false;
			option = PropertyUtils.describe(method);
			// Add fields to comply with new shipping rules
			option.put("processorClass", "BasicShippingProcessor");
			// Shows in bold like UPS/Fedex before the method name.
			option.put("processorName", "Basic");
			option.put("shippingMethodObject", method);
			// End new fields
			String estimatedDeliveryDate = doEstimatedDeliveryDateWithShipDateToday(method
					.getNumberDaysToShip());
			if (estimatedDeliveryDate != null) {
				option.put("DeliveryDate", estimatedDeliveryDate);
			}
			if (method.getBase() != null) {
				rawPrice = method.getBase().doubleValue();
			}
			if (method.getMinimum() != null) {
				minimum = method.getMinimum().doubleValue();
			}
			if (method.getMethodType().equals("flatRate")) {
				priceFound = true;
			} else if (method.getShippingRates() != null) {
				Iterator j = method.getShippingRates().iterator();
				while (j.hasNext()) {
					ShippingRate rate = (ShippingRate) j.next();
					if (log.isDebugEnabled())
						log.debug("floor: " + rate.getFloor());
					if (log.isDebugEnabled())
						log.debug("ceiling: " + rate.getCeiling());
					if (log.isDebugEnabled())
						log.debug("amount: " + rate.getAmount());
					if (method.getMethodType().equals("perWeightUnitTable")) {
						// Shipping = order's weight times rate amount
						if (totalWeight >= rate.getFloor().doubleValue()
								&& (totalWeight <= rate.getCeiling()
										.doubleValue() || rate.getCeiling()
										.doubleValue() == 0)) {
							rawPrice = rawPrice + totalWeight
									* rate.getAmount().doubleValue();
							priceFound = true;
							break;
						}
					} else if (method.getMethodType().equals("weightTable")) {
						// Shipping = rate amount corresponding to order's
						// weight
						if (totalWeight >= rate.getFloor().doubleValue()
								&& (totalWeight <= rate.getCeiling()
										.doubleValue() || rate.getCeiling()
										.doubleValue() == 0)) {
							rawPrice = rawPrice
									+ rate.getAmount().doubleValue();
							priceFound = true;
							break;
						}
					} else if (method.getMethodType().equals("priceTable")) {
						// Shipping = rate amount corresponding to order's
						// subtotal
						if (getUser().getOrder() != null
								&& getUser().getOrder().getSubtotal() != null
								&& getUser().getOrder().getSubtotal()
										.doubleValue() >= rate.getFloor()
										.doubleValue()
								&& (getUser().getOrder().getSubtotal()
										.doubleValue() <= rate.getCeiling()
										.doubleValue() || rate.getCeiling()
										.doubleValue() == 0)) {
							rawPrice = rawPrice
									+ rate.getAmount().doubleValue();
							priceFound = true;
							break;
						}
					} else if (method.getMethodType().equals("quantityTable")) {
						// Shipping = rate amount corresponding to order's
						// quantity
						if (getUser().getOrder() != null
								&& getUser().getOrder().getQuantity() != null
								&& getUser().getOrder().getQuantity()
										.intValue() >= rate.getFloor()
										.doubleValue()
								&& (getUser().getOrder().getQuantity()
										.intValue() <= rate.getCeiling()
										.doubleValue() || rate.getCeiling()
										.doubleValue() == 0)) {
							rawPrice = rawPrice
									+ rate.getAmount().doubleValue();
							priceFound = true;
							break;
						}
					} else if (method.getMethodType().equals(
							"cumulativeQuantityTable")) {
						// Shipping = rate amount times order's quantity for
						// each range in table
						if (getUser().getOrder() != null
								&& getUser().getOrder().getQuantity() != null
								&& getUser().getOrder().getQuantity()
										.intValue() >= rate.getFloor()
										.doubleValue()) {
							if (getUser().getOrder().getQuantity().intValue() >= rate
									.getCeiling().doubleValue()
									&& rate.getCeiling().doubleValue() != 0) {
								rawPrice = rawPrice
										+ ((rate.getCeiling().doubleValue()
												- rate.getFloor().doubleValue() + 1) * rate
												.getAmount().doubleValue());
								priceFound = true;

							} else if (getUser().getOrder().getQuantity() != null) {
								rawPrice = rawPrice
										+ ((getUser().getOrder().getQuantity()
												.intValue()
												- rate.getFloor().doubleValue() + 1) * rate
												.getAmount().doubleValue());
								priceFound = true;
							} else {
								rawPrice = 0;
							}
						}
					}
				}
			}
			if (priceFound && option != null) {
				String currencyCode;
				try {
					currencyCode = (String) getSettings().getValue(
							"currencyCode");
				} catch (Exception e) {
					if (log.isDebugEnabled())
						log
								.debug("Couldn't get currencyCode setting, using USD.");
					currencyCode = "USD";
				}
				if (minimum > rawPrice) {
					option.put("priceDouble", new Double(minimum));
					option.put("price", formatPrice(minimum, currencyCode,
							getUser().getLocale()));
					option.put("originalPriceDouble", new Double(minimum));
					option.put("originalPrice", formatPrice(minimum,
							currencyCode, getUser().getLocale()));
					option.put("currencyCode", currencyCode);
					Collection discounts = new ArrayList();
					option.put("discounts", discounts);
				} else {
					option.put("priceDouble", new Double(rawPrice));
					option.put("price", formatPrice(rawPrice, currencyCode,
							getUser().getLocale()));
					option.put("originalPriceDouble", new Double(rawPrice));
					option.put("originalPrice", formatPrice(rawPrice,
							currencyCode, getUser().getLocale()));
					option.put("currencyCode", currencyCode);
					Collection discounts = new ArrayList();
					option.put("discounts", discounts);
				}
				if (log.isDebugEnabled())
					log.debug("price: " + option.get("price"));
				options.add(option);
			}
		}
		return options;
	}
}
