package com.softslate.commerce.businessobjects.tax;

import java.util.Collection;
import java.util.Iterator;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.daos.tax.BasicTaxRateGatewayDAO;

/**
 * Processes taxes in the store for a given user's cart. This implementation
 * looks up the tax rates configured in the <code>sscBasicTaxRates</code> table,
 * which provides for the ability to set up a tax table based on country, state,
 * city, and postal code.
 * <p>
 * <code>BasicTaxProcessor</code> is the default implementation of the
 * <code>TaxProcessor</code> interface for the application.
 * 
 * @author David Tobey
 */
public class BasicTaxProcessor extends BaseTaxProcessor implements TaxProcessor {

	static Log log = LogFactory.getLog(BasicTaxProcessor.class);

	public int processTax() throws Exception {
		log.info("Processing tax.");
		if (getUser().getOrder() == null || getUser().getOrder().firstDelivery() == null) {
			log.info("Order or delivery not defined.");
			return 1;
		}
		log.info("Taxable Subtotal: " + getUser().getOrder().firstDelivery().getTaxableSubtotal());

		// Figure out the tax rate that applies to this order
		double applicableTaxRate = computeApplicableTaxRate(getUser().getOrder().firstDelivery().getCountry(),
				getUser().getOrder().firstDelivery().getState(), getUser().getOrder().firstDelivery().getCity(),
				getUser().getOrder().firstDelivery().getPostalCode());

		// areTaxRatesPercentages - true if the values are stored like "8.00" for 8%, false if the values are stored
		// like "0.08".
		String areTaxRatesPercentages = getSettings().getValue("areTaxRatesPercentages");
		if (areTaxRatesPercentages == null || areTaxRatesPercentages.equals("1")) {
			applicableTaxRate = applicableTaxRate / 100;
		}

		// Compute the tax and round the tax amount to two decimal places.
		double taxAmount = applicableTaxRate * getUser().getOrder().firstDelivery().getTaxableSubtotal().doubleValue();
		getUser().getOrder().firstDelivery().setTax(utils().roundForMoney(taxAmount));
		log.info("Applicable tax rate: " + applicableTaxRate + "; tax computed: " + taxAmount);
		return 0;
	}

	public double computeApplicableTaxRate(String deliveryCountry,
			String deliveryState, String deliveryCity, String deliveryPostalCode)
			throws Exception {
		if (StringUtils.isNotBlank(deliveryPostalCode) && deliveryPostalCode.length() > 5
				&& StringUtils.isNotBlank(deliveryCountry) & deliveryCountry.equals("US")) {
			deliveryPostalCode = deliveryPostalCode.substring(0, 5);
		}
		double applicableTaxRate = 0;
		String areTaxRatesCumulative = (String) getSettings().getValue(
				"areTaxRatesCumulative");
		boolean accumulate = areTaxRatesCumulative.equals("1");
		int matchSpecificity = 0;
		Collection taxRates = loadActiveBasicTaxRates();
		int previousRateID = 0;
		Iterator i = taxRates.iterator();
		while (i.hasNext()) {
			BasicTaxRate rate = (BasicTaxRate) i.next();
			if (rate == null || previousRateID == rate.getBasicTaxRateID() || rate.getCountry() == null
					|| rate.getRate() == null) {
				continue;
			}
			String countryCode = rate.getCountry().getCode();
			String stateCode = null;
			if (rate.getState() != null) {
				stateCode = rate.getState().getCode();
			}
			String city = rate.getCity();
			String postalCode = rate.getPostalCode();
			double rateDouble = rate.getRate().doubleValue();
			boolean countryExactMatch = deliveryCountry != null
					&& deliveryCountry.equals(countryCode);
			boolean stateExactMatch = deliveryState != null
					&& deliveryState.equals(stateCode);
			boolean cityExactMatch = deliveryCity != null
					&& deliveryCity.equals(city);
			boolean postalCodeExactMatch = deliveryPostalCode != null
					&& deliveryPostalCode.equals(postalCode);
			boolean countryBroadMatch = countryExactMatch
					|| countryCode == null || countryCode.equals("");
			boolean stateBroadMatch = stateExactMatch || stateCode == null
					|| stateCode.equals("");
			boolean cityBroadMatch = cityExactMatch || city == null
					|| city.equals("");
			boolean postalCodeBroadMatch = postalCodeExactMatch
					|| postalCode == null || postalCode.equals("");
			if (accumulate) {
				if (countryBroadMatch && stateBroadMatch && cityBroadMatch
						&& postalCodeBroadMatch) {
					if (log.isDebugEnabled())
						log.debug("accumulating: " + rateDouble);
					if (log.isDebugEnabled())
						log.debug("countryCode: " + countryCode);
					if (log.isDebugEnabled())
						log.debug("stateCode: " + stateCode);
					if (log.isDebugEnabled())
						log.debug("city: " + city);
					if (log.isDebugEnabled())
						log.debug("postalCode: " + postalCode);
					applicableTaxRate = applicableTaxRate + rateDouble;
				}
			} else {
				if (countryBroadMatch && stateBroadMatch && cityBroadMatch
						&& postalCodeExactMatch) {
					applicableTaxRate = rateDouble;
					matchSpecificity = 4;
					if (log.isDebugEnabled())
						log.debug("postalCodeExactMatch");
					break;
				} else if (countryBroadMatch && stateBroadMatch
						&& cityExactMatch && postalCodeBroadMatch) {
					applicableTaxRate = rateDouble;
					matchSpecificity = 3;
					if (log.isDebugEnabled())
						log.debug("stateExactMatch");
				} else if (matchSpecificity < 2 && countryBroadMatch
						&& stateExactMatch && cityBroadMatch
						&& postalCodeBroadMatch) {
					applicableTaxRate = rateDouble;
					matchSpecificity = 2;
					if (log.isDebugEnabled())
						log.debug("countryExactMatch");
				} else if (matchSpecificity < 1 && countryExactMatch
						&& stateBroadMatch && cityBroadMatch
						&& postalCodeBroadMatch) {
					applicableTaxRate = rateDouble;
					matchSpecificity = 1;
					if (log.isDebugEnabled())
						log.debug("postalCodeExactMatch");
				}
			}
		}
		log.debug("applicableTaxRate: " + applicableTaxRate);
		return applicableTaxRate;
	}

	public Collection loadActiveBasicTaxRates() throws Exception {
		BasicTaxRateGatewayDAO dao =  getDaoFactory()
				.createDAO(BasicTaxRateGatewayDAO.class);
		return dao.loadActiveBasicTaxRates();
	}

}
