package com.softslate.commerce.customer.order;

import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionServlet;

import com.softslate.commerce.businessobjects.payment.PaymentProcessor;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Interface for utility methods used by Struts Action classes related to the checkout process. Note that some Administrator functions involved in editing orders use these methods as well.
 * <p>
 * When the application needs to create an instance that implements
 * <code>CheckoutUtils</code>, it calls
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory#createUtilsObject} ,
 * which finds the name of the Java class to instantiate from the
 * "checkoutUtilsImplemeter" setting in the <code>appComponents.properties</code>
 * file.
 * <p>
 * The default "checkoutUtilsImplemeter" is
 * {@link com.softslate.commerce.customer.order.CheckoutUtilsImpl}.
 * 
 * @author David Tobey
 * 
 */
public interface CheckoutUtils {

	/**
	 * Determines what checkout screen to direct the user to based on the status of the session and the store's database
	 * configuration. The possible screen names returned are:
	 * <ul>
	 * <li>"catalogMode", if the store's "catalogMode" database setting is set.</li>
	 * <li>"cart", if the user's cart is empty.</li>
	 * <li>"forceLogin", if the "forceLogin" database setting is set and the user is not logged in.</li>
	 * <li>"inviteLogin", if the "inviteLogin" database setting is set and the user has not declined to log in and is
	 * not logged in.</li>
	 * <li>"checkoutAddresses", if the user has not entered in all the required address fields for the order, and the
	 * "useComboScreen" setting is not set.</li>
	 * <li>"checkoutPayment", if payment information is required, the total of the current order is greater than zero,
	 * and the "useComboScreen" setting is not set.</li>
	 * <li>"checkoutCombo", if the user has not entered in all the required address fields, or if payment information is
	 * required, and if the "useComboScreen" setting is set.</li>
	 * <li>"checkoutConfirm", if none of the other screens apply.</li>
	 * </ul>
	 * <p>
	 * The above screen names correspond to <code>ActionForward</code>s defined in the
	 * "/WEB-INF/conf/order/struts-config-order.xml" file.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds various properties of the request to help
	 *            action classes communicate with the rest of the application.
	 * @return A string identifying the name of the checkout screen to direct the user to.
	 * @throws Exception
	 */
	abstract String findCheckoutScreen(BaseForm baseForm) throws Exception;

	/**
	 * Validates the required billing address fields in the user's cart. Loops
	 * through the "billingRequired" database setting to check which fields are
	 * required.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @return <code>true</code> if all the required billing fields are
	 *         present in the user's cart. <code>false</code> if not.
	 * @throws Exception
	 */
	abstract boolean validateBillingFields(BaseForm baseForm) throws Exception;

	/**
	 * Validates the required delivery address fields in the user's cart. Loops
	 * through the "deliveryRequired" database setting to check which fields are
	 * required.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @return <code>true</code> if all the required billing fields are
	 *         present in the user's cart. <code>false</code> if not.
	 * @throws Exception
	 */
	abstract boolean validateDeliveryFields(BaseForm baseForm) throws Exception;

	/**
	 * Initializes an instance of <code>LoginForm</code> and populates it from
	 * request parameters. Used on the invite login screen and the force login
	 * screen.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @throws Exception
	 */
	abstract void prepareLoginForm(BaseForm baseForm, ActionServlet servlet)
			throws Exception;

	/**
	 * Initializes an instance of <code>RegisterForm</code> and populates it
	 * from request parameters. Used on the checkout register screen.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @throws Exception
	 */
	abstract void prepareRegisterForm(BaseForm baseForm, ActionServlet servlet)
			throws Exception;

	/**
	 * Processes a login request during the checkout process. Used when a user
	 * logs in from the invite login screen or the force login screen.
	 * <p>
	 * The method runs
	 * {@link com.softslate.commerce.businessobjects.customer.CustomerProcessor#processLogin(Map)}
	 * to process the login request.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @return An <code>ActionForward</code> corresponding to the results of
	 *         the processing. The <code>ActionForward</code> will be named
	 *         "failureInvalidLogin" or "success".
	 * @throws Exception
	 */
	abstract ActionForward processCheckoutLogin(BaseForm baseForm,
			HttpServletResponse response) throws Exception;

	/**
	 * Processes a request to register a new customer account during the
	 * checkout process. Used when a user registers from the checkout register
	 * screen.
	 * <p>
	 * This method runs
	 * {@link com.softslate.commerce.businessobjects.customer.CustomerProcessor#processRegister(Map)}
	 * to process the register request.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @return An <code>ActionForward</code> corresponding to the results of
	 *         the processing. The <code>ActionForward</code> will be named
	 *         "failureUserName" if the user name is already taken, or
	 *         "success".
	 * @throws Exception
	 */
	abstract ActionForward processCheckoutRegister(BaseForm baseForm)
			throws Exception;

	/**
	 * Processes a request to continue the checkout process without logging in
	 * or creating an account. Used when a user declines to log in from the
	 * invite login screen.
	 * <p>
	 * This method places an attribute in the user's session named
	 * "declinedLoginBeforeCheckout".
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @return An <code>ActionForward</code> corresponding to the results of
	 *         the processing. Always returns the "success"
	 *         <code>ActionForward</code>.
	 * @throws Exception
	 */
	abstract ActionForward processDeclineLogin(BaseForm baseForm)
			throws Exception;

	/**
	 * Loads the active shipping options for display to the user. Used on the
	 * checkout addresses screen. Loops through the classes in the
	 * "activeShippingProcessors" database setting and invokes the
	 * <code>loadShippingOptions</code> method in each to get the available
	 * options.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @return A <code>Collection</code> containing each of the shipping
	 *         option available for the user to select.
	 * @throws Exception
	 * @see com.softslate.commerce.businessobjects.shipping.ShippingProcessor#loadShippingOptions(Map)
	 */
	abstract Collection loadShippingOptions(BaseForm baseForm) throws Exception;

	/**
	 * Initializes an instance of <code>CheckoutAddressesForm</code> with
	 * address and shipping option information. Used on the checkout addresses
	 * screen. If the current user is a logged in customer, populates the form
	 * with the account's address information.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @throws Exception
	 */
	abstract void prepareCheckoutAddressesForm(CheckoutAddressesForm baseForm)
			throws Exception;

	abstract void prepareCheckoutAddressesAddressBookForm(
			CheckoutAddressesForm baseForm) throws Exception;

	/**
	 * Initializes an instance of <code>CheckoutAddressesForm</code> with
	 * address and shipping option information. Used on the checkout addresses
	 * screen. If the current user is a logged in customer, populates the form
	 * with the account's address information.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @throws Exception
	 */
	abstract void prepareCheckoutShippingForm(BaseForm baseForm)
			throws Exception;

	/**
	 * Processes submissions to record addresses in the current user's cart.
	 * Used for requests to "/CheckoutAddresses.do".
	 * <p>
	 * Invokes
	 * {@link com.softslate.commerce.businessobjects.order.CartProcessor#processCheckoutAddresses(Map)}
	 * to process the submitted address information. Then invokes
	 * {@link #processInventoryDiscountCheck(BaseForm)} to perform a check on
	 * inventory and discounts for items in the cart.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @return An <code>ActionForward</code> corresponding to the results of
	 *         the processing. The <code>ActionForward</code> will be named
	 *         "failureItemsAdjusted", "failureAllItemsRemoved", or "success".
	 * @throws Exception
	 */
	abstract ActionForward processCheckoutAddresses(BaseForm baseForm)
			throws Exception;

	/**
	 * Processes submissions to record addresses in the current user's cart.
	 * Used for requests to "/CheckoutAddresses.do".
	 * <p>
	 * Invokes
	 * {@link com.softslate.commerce.businessobjects.order.CartProcessor#processCheckoutAddresses(Map)}
	 * to process the submitted address information. Then invokes
	 * {@link #processInventoryDiscountCheck(BaseForm)} to perform a check on
	 * inventory and discounts for items in the cart.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @param response
	 *            The HttpServletResponse object associated with the request. If
	 *            present and the "autoSavedCartToken" cookie is present, that
	 *            cookie will be removed so no personal information is
	 *            retrievable through the cookie.
	 * @return An <code>ActionForward</code> corresponding to the results of
	 *         the processing. The <code>ActionForward</code> will be named
	 *         "failureItemsAdjusted", "failureAllItemsRemoved", or "success".
	 * @throws Exception
	 */
	abstract ActionForward processCheckoutAddresses(BaseForm baseForm,
			HttpServletResponse response) throws Exception;

	/**
	 * Performs a check on the inventory and processes discounting for the
	 * current user's cart. Makes sure that if products are to be decremented on
	 * checkout, no skus are out of stock.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @return A Map containing a key named resultCode: 0 indicates no changes,
	 *         1 indicates the cart was adjusted, 2 indicates all order items
	 *         were removed (due to inventory being out of stock).
	 * @throws Exception
	 */
	abstract Map processInventoryDiscountCheck(BaseForm baseForm)
			throws Exception;

	/**
	 * Processes submissions to record shipping options in the current user's
	 * cart. Used for requests to "/CheckoutShipping.do".
	 * <p>
	 * Invokes {@link #processShipping(BaseForm)} to process shipping charges
	 * and {@link #processTax(BaseForm)} to process tax charges.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @return An <code>ActionForward</code> corresponding to the results of
	 *         the processing. The <code>ActionForward</code> will be named
	 *         "failureShippingProcess", "failureTaxProcess", "success",
	 *         "successNoPaymentRequired", or "successNoPaymentRequiredConfirm".
	 * @throws Exception
	 */
	abstract ActionForward processCheckoutShipping(BaseForm baseForm)
			throws Exception;

	/**
	 * Processes shipping charges for the current user's cart. Called when the
	 * checkout addresses form is processed.
	 * <p>
	 * Goes through the classes in the "activeShippingProcessors" database
	 * setting and invokes the <code>processShipping(Map)</code> method in
	 * each to process the charges. The incoming <code>baseForm</code> should
	 * have a property named <code>shippingOption</code> containing the
	 * shipping option selected by the user.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @return A Map with a key named resultCode, which is 0, if the processing
	 *         was successful. A non-zero integer indicates there was a problem
	 *         and further processing should stop.
	 * @throws Exception
	 * @see com.softslate.commerce.businessobjects.shipping.ShippingProcessor#processShipping(Map)
	 */
	abstract Map processShipping(BaseForm baseForm) throws Exception;

	/**
	 * 
	 * Processes tax charges for the current user's cart. Called when the
	 * checkout addresses form is processed.
	 * <p>
	 * Goes through the classes in the "activeTaxProcessors" database setting
	 * and invokes the <code>processTax(Map)</code> method in each to process
	 * the charges.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @return A Map with a key named resultCode, which is 0, if the processing
	 *         was successful. A non-zero integer indicates there was a problem
	 *         and further processing should stop.
	 * @throws Exception
	 */
	abstract Map processTax(BaseForm baseForm) throws Exception;

	/**
	 * Creates payment form instances and populates them from the request
	 * parameters. Used on the checkout payment screen so each of the active
	 * payment forms can be displayed. Loops through the "activePaymentForms"
	 * database setting to find the class names of the payment forms to create.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @throws Exception
	 */
	abstract void preparePaymentForms(BaseForm baseForm, ActionServlet servlet)
			throws Exception;

	/**
	 * Prepares a given payment form by populating its properties from the
	 * request's parameters.
	 * 
	 * @param paymentForm
	 *            The payment form being populated from the request.
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @throws Exception
	 */
	abstract void preparePaymentForm(ActionForm paymentForm, BaseForm baseForm,
			ActionServlet servlet) throws Exception;

	/**
	 * Processes submissions of payment information by the current user. Used
	 * for requests to "/CheckoutPayment.do".
	 * <p>
	 * Loops through each of the classes in the "activePaymentForms" and
	 * "activePaymentProcessors" database settings to find the payment form that
	 * was used to submit payment information, and its corresponding
	 * {@link com.softslate.commerce.businessobjects.payment.PaymentProcessor}.
	 * Uses the "paymentFormCount" request parameter to determine which form was
	 * submitted.
	 * <p>
	 * Invokes {@link #validatePaymentForm(ActionForm, BaseForm)} to validate
	 * the submission, and if valid,
	 * {@link #processPayment(PaymentProcessor, ActionForm, BaseForm)} to
	 * process it.
	 * <p>
	 * If a confirmation screen must be displayed, this method places the
	 * payment form and the name of the payment processor in the customer's
	 * session, so that the <code>processCheckoutConfirm</code> method can
	 * retrieve them.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @return An <code>ActionForward</code> corresponding to the results of
	 *         the processing. The <code>ActionForward</code> will be named
	 *         "failureFormValidation", "failurePaymentProcess", "success", or
	 *         "successCheckoutConfirm".
	 * @throws Exception
	 */
	abstract ActionForward processCheckoutPayment(BaseForm baseForm,
			ActionServlet servlet) throws Exception;

	/**
	 * Validates a given payment form for a given request. Used when a
	 * submission is made from the checkout payment screen to validate the
	 * submission of payment information. The method simply runs the
	 * <code>validate</code> method of the payment form, saving any validation
	 * errors to the request.
	 * 
	 * @param paymentForm
	 *            The payment form that was used by the user to submit payment
	 *            information.
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @return <code>true</code> if the submission passes validation.
	 *         <code>false</code> if not.
	 * @throws Exception
	 */
	abstract boolean validatePaymentForm(ActionForm paymentForm,
			BaseForm baseForm) throws Exception;

	/**
	 * Processes payment information for a request with a given payment
	 * processor. Used when a submission is made from the checkout payment
	 * screen to process payment information.
	 * <p>
	 * This method runs the <code>processPayment(Map)</code> method of the
	 * incoming <code>PaymentProcessor</code>, which returns a Map that
	 * includes a key named resultCode. This method returns the int value of
	 * that resultCode.
	 * <p>
	 * If the resultCode is not equal to zero, it indicates an error in the
	 * processing of the payment. This method will look up an error message in
	 * the "/WEB-INF/classes/resources/payment/application.properties" file
	 * corresponding to the resultCode and the name of the payment processor.
	 * For example, the error message for the message key
	 * <code>payment.errorPayflowProProcessor10000</code> will be placed in
	 * the request, if the payment processor was PayflowProProcessor and the
	 * resultCode was 10000. Additional error messages that follow the same
	 * naming pattern can be placed in the
	 * "/WEB-INF/classes/resources/payment/application.properties" file.
	 * 
	 * @param paymentProcessor
	 *            The <code>PaymentProcessor</code> corresponding to the
	 *            payment form used by the user to submit payment information.
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @return <code>true</code> if the submission passes validation.
	 *         <code>false</code> if not.
	 * @throws Exception
	 */
	abstract int processPayment(PaymentProcessor paymentProcessor,
			ActionForm paymentForm, BaseForm baseForm) throws Exception;

	/**
	 * Processes a request to confirm an order. Processes the payment form
	 * stored in the session with the session's payment processor. If processing
	 * fails, a failure is returned. Otherwise, the
	 * <code>processOrderComplete</code> method is run.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @throws Exception
	 */
	abstract ActionForward processCheckoutConfirm(BaseForm baseForm)
			throws Exception;

	/**
	 * Invoked when a order has just been completed. Decrements inventory if
	 * called for, and places the current <code>User</code> object, as well as
	 * the <code>baseForm</code> object in the request scope, for use by the
	 * JSP templates that display the invoice and send out the notification and
	 * confirmation emails.
	 * 
	 * @param baseForm
	 *            The form class corresponding to the current request. Holds
	 *            various properties of the request to help action classes
	 *            communicate with the rest of the application.
	 * @throws Exception
	 */
	abstract Map processOrderComplete(BaseForm baseForm) throws Exception;

}
