package com.softslate.commerce.daos.core;

import java.sql.Connection;
import java.util.Collection;
import java.util.Date;
import java.util.Map;
import java.util.Properties;

import javax.sql.DataSource;

import org.hibernate.Filter;

import com.google.inject.Injector;
import com.softslate.commerce.businessobjects.core.Settings;

/**
 * A parent interface for all of the other interfaces in the data access layer.
 * <p>
 * The methods provided allow implementations to communicate with other elements
 * of the application. In particular, retrieve settings (using
 * {@link #getSettings()}), retrieve database connections (using
 * {@link #getConnection()}), retrieve SQL statements (using
 * {@link #getSqlMap()}), and create other data access objects (using
 * {@link #getDaoFactory()}).
 * 
 * @author David Tobey
 * 
 */
public interface DAOInterface {
    /**
     * Initializes this data access object by copying incoming parameters into
     * its properties. This method is used by
     * {@link com.softslate.commerce.daos.core.DAOFactory} after object
     * creation to copy the application's
     * {@link com.softslate.commerce.daos.core.DAOFactory} and
     * {@link com.softslate.commerce.businessobjects.core.Settings}
     * into the new object. The <code>sqlMap</code> and
     * <code>dataSource</code> properties are also copied from the
     * <code>DAOFactory</code>.
     * 
     * @param transferMap
     *            A <code>Map</code> containing objects such as the
     *            application's <code>DAOFactory</code> and
     *            <code>Settings</code>.
     * @throws Exception
     */
    abstract void initialize(Map transferMap) throws Exception;

    /**
     * Retrieves the <code>Settings</code> previously set for this data access
     * object.
     * 
     * @return A <code>Settings</code> object that represents various settings
     *         for the system.
     */
    abstract Settings getSettings();

    abstract void setSettings(Settings settings);

    /**
     * Retrieves this factory's component settings. The component settings
     * identify which Java class implement the various Interfaces used by the
     * system.
     */
    public Properties getAppComponents();

    public void setAppComponents(Properties appComponents);

    /**
     * Retrieves this factory's application settings. The current application
     * settings for the system. This factory will pass the application settings
     * to every DAO it creates. The application settings are settings for
     * information unique to a given installation, such as the location of files
     * on the server, as opposed to the <code>SettingsBean</code>, which
     * stores settings drawn from the database.
     */
    public Properties getAppSettings();

    public void setAppSettings(Properties appSettings);

    /**
     * Retrieves a <code>Connection</code> to the application's
     * <code>DataSource</code>.
     * 
     * @return A <code>Connection</code> to the application's
     *         <code>DataSource</code>.
     */
    abstract Connection getConnection();

    abstract void setConnection(Connection connection);

    /**
     * Retrieves the application's <code>DataSource</code>.
     * 
     * @return The application's <code>DataSource</code>.
     */
    abstract DataSource getDataSource();

    abstract void setDataSource(DataSource dataSource);

    /**
     * Retrieves the application's <code>DAOFactory</code>.
     * 
     * @return The application's <code>DAOFactory</code>.
     */
    abstract DAOFactory getDaoFactory();

    abstract void setDaoFactory(DAOFactory daoFactory);

    /**
     * Retrieves a <code>Map</code> of the SQL statements used by the
     * application.
     * 
     * @return A <code>Map</code> of the SQL statements used by the
     *         application.
     */
    abstract Map getSqlMap();

    abstract void setSqlMap(Map sqlMap);

	/**
	 * Retrieves the <code>Injector</code> previously set.
	 * 
	 * @return A <code>Injector</code> object that can be used to create other
	 *         objects and inject them with dependencies.
	 */
	public Injector getInjector();

	/**
	 * Sets this object's <code>Injector</code> that can be used to create other
	 * objects and inject them with dependencies.
	 * 
	 * @param injector
	 *            The current <code>Injector</code> for the system, which can be
	 *            used to create other objects and inject them with
	 *            dependencies.
	 */
	public void setInjector(Injector injector);

    /**
     * Retrieves a utility object that provides useful methods for daos.
     * 
     * @return An implementation of DAOUtils.
     */
    abstract DAOUtils getDAOUtils();

    abstract void setDAOUtils(DAOUtils daoUtils);
    
	public Object loadObject(Object object) throws Exception;
	
	public Object updateObject(Object object, boolean commit) throws Exception;
	
	public boolean deleteObject(Object object, boolean commit) throws Exception;
	
	public Object insertObject(Object object, boolean commit) throws Exception;
	
	public Collection loadAll() throws Exception;
	
	public Collection loadAll(Filter filter) throws Exception;

	public String formatDateTime(Date dateTime);

	public Date parseDateTime(String dateTime) throws Exception;
}
