package com.softslate.commerce.daos.core;

import java.io.IOException;
import java.io.Writer;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collection;
import java.util.Map;

public interface DAOUtils {

	/**
	 * An array of strings to eliminate from database search queries.
	 */
	abstract String[] getStopWords();

	/**
	 * Given an array of strings, returns a single string that can be used as a
	 * list in an SQL expression. Each of the elements of the incoming array are
	 * separated by commas.
	 * 
	 * @param stringArray
	 *            The array of strings to convert into a comma-separated list.
	 * @return A string that can be used as a list in an SQL expression.
	 */
	abstract String getList(String[] stringArray);

	/**
	 * Given an array of strings, returns a single string that can be used as a
	 * list in an SQL expression. Each of the elements of the incoming array are
	 * quoted and separated by commas.
	 * 
	 * @param stringArray
	 *            The array of strings to convert into a quoted string.
	 * @return A string that can be used as a list in an SQL expression.
	 */
	abstract String getQuotedList(String[] stringArray);

	/**
	 * Given a ResultSet and a field name, returns a single string that can be
	 * used as a list in an SQL expression. Each of the elements of the incoming
	 * ResultSet corresponding to the given field are separated by commas.
	 * Helpful for older databases that do not support sub queries.
	 * 
	 * @param resultSet
	 *            The ResultSet holding the values.
	 * @param field
	 *            The name of the field whose values are to be returned in a
	 *            list.
	 * @return A string that can be used as a list in an SQL expression
	 * @throws SQLException
	 */
	abstract String getListFromResultSet(ResultSet resultSet, String field)
			throws SQLException;

	/**
	 * Given a Collection of Maps representing a ResultSet, and a field name for
	 * a field, returns a String list representing each field's value in the
	 * incoming Collection.
	 * 
	 * @param collection
	 *            The Collection holding the values.
	 * @return A string that can be used as a list in an SQL expression.
	 */
	abstract String getListFromCollection(Collection collection, String field);

	/**
	 * Given a Collection of Strings, returns a String list representing each
	 * element's value in the incoming Collection.
	 * 
	 * @param collection
	 *            The Collection holding the values.
	 * @return A string that can be used as a list in an SQL expression.
	 */
	abstract String getListFromStringCollection(Collection collection);

	/**
	 * Given a ResultSet and a field name for a field, returns a Collection of
	 * Strings representing each field value in the ResultSet.
	 * 
	 * @param resultSet
	 *            The ResultSet holding the values.
	 * @param field
	 *            The name of the field whose values are to be returned in a
	 *            Colleciton.
	 * @return A Collection of Strings representing each field value in the
	 *         ResultSet
	 * @throws SQLException
	 */
	abstract Collection getCollectionFromResultSet(ResultSet resultSet,
			String field) throws SQLException;

	/**
	 * Given a Collection of Maps representing a ResultSet, and a field name for
	 * a field, returns a Collection of Strings representing each field's value
	 * in the incoming Collection.
	 * 
	 * @param collection
	 *            The Collection holding the values.
	 * @param field
	 *            The name of the field whose values are to be returned in a
	 *            Collection.
	 * @return A Collection of Strings representing each field's value in the
	 *         incoming Collection
	 */
	abstract Collection getCollectionFromCollection(Collection collection,
			String field);

	/**
	 * Given a Collection of beans representing a ResultSet, and a field name
	 * for a field, returns a Collection of Integers representing each field's
	 * value in the incoming Collection.
	 * 
	 * @param collection
	 *            The Collection holding the values.
	 * @param field
	 *            The name of the field whose values are to be returned in a
	 *            Collection.
	 * @return a Collection of Integers
	 * @throws Exception
	 */
	abstract Collection getCollectionFromBeans(Collection collection,
			String field) throws Exception;

	/**
	 * Given a Collection of Strings, returns a collection of Integers
	 * corresponding to the strings.
	 * 
	 * @param collection
	 *            A collection of Strings
	 * @return A collection of Integers
	 */
	abstract Collection getIntegersFromStrings(Collection collection);

	/**
	 * Retrieves a string indicating the number of rows and columns returned
	 * from a <code>ResultSet</code>.
	 * 
	 * @param resultSet
	 *            The <code>ResultSet</code> to produce a debugging message
	 *            for.
	 * @return A string indicating the number of rows and columns returned from
	 *         a <code>ResultSet</code>.
	 * @throws SQLException
	 */
	abstract String getDebugMessage(ResultSet resultSet) throws SQLException;

	/**
	 * Given a <code>Map</code> of string keys, returns a single string that
	 * can be used as a list in an SQL expression. Each of the elements of the
	 * incoming <code>Map</code> are quoted and separated by commas.
	 * 
	 * @param map
	 *            The <code>Map</code> to convert into a quoted string.
	 * 
	 * @return A string that can be used as a list in an SQL expression.
	 * @throws Exception
	 */
	abstract String getQuotedList(Map map) throws Exception;

	/**
	 * Outputs a matrix of strings as comma-separated values.
	 * 
	 * This implementation encloses problem values in double-quotes, prepends
	 * backslash to embedded double-quotes and backslashes, and represents
	 * embedded newlines as backslash-n.
	 * 
	 * @param data
	 *            List of String arrays or Object arrays.
	 * @param out
	 *            The stream to write to.
	 */
	abstract void outputCsv(Collection data, Writer out) throws IOException;

	/**
	 * Prepares a string for insertion into an SQL search expression. Adds % to
	 * the beginning and end of the string and in between each word. Also strips
	 * the string of characters dangerous to an SQL search expression.
	 * 
	 * @param searchString
	 * @return A string acceptable for a LIKE expression.
	 */
	abstract String prepareSearchString(String searchString);

	/**
	 * Encrypts a string using the given encryption type.
	 * 
	 * @param stringToEncrypt
	 *            The String to encrypt.
	 * @param encryptionType
	 *            "noEncryption", "keyPairStore", "keyPairNoStore", or "oneWay".
	 * @return The encrypted string
	 */
	abstract String getEncrypted(String stringToEncrypt, String encryptionType,
			String publicKeyFile);

	public String getEncrypted(String stringToEncrypt, String encryptionType, String publicKeyFile,
			String oneWayAlgorithm);

	/**
	 * Decrypts a string using the given encryption type.
	 * 
	 * @param stringToDecrypt
	 *            The String to decrypt.
	 * @param encryptionType
	 *            "noEncryption", "keyPairStore", "keyPairNoStore", or "oneWay".
	 * @return The decrypted string
	 */
	abstract String getDecrypted(String stringToDecrypt, String encryptionType,
			String privateKeyFile);

	abstract void generateTwoWayKey(String keyFile) throws Exception;

	abstract String generateKeyPair(String publicFile, String privateFile,
			boolean savePrivate) throws Exception;

}
